"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from flexprice import models, utils
from flexprice._hooks import HookContext
from flexprice.types import OptionalNullable, UNSET
from flexprice.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional, Union


class CreditNotes(BaseSDK):
    def create_credit_note(
        self,
        *,
        invoice_id: str,
        reason: models.CreditNoteReason,
        credit_note_number: Optional[str] = None,
        idempotency_key: Optional[str] = None,
        line_items: Optional[
            Union[
                List[models.DtoCreateCreditNoteLineItemRequest],
                List[models.DtoCreateCreditNoteLineItemRequestTypedDict],
            ]
        ] = None,
        memo: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        process_credit_note: Optional[bool] = True,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoCreditNoteResponse:
        r"""Create credit note

        Use when issuing a refund or adjustment (e.g. customer dispute or proration). Links to an invoice; create as draft then finalize.

        :param invoice_id: invoice_id is the unique identifier of the invoice this credit note is applied to
        :param reason:
        :param credit_note_number: credit_note_number is an optional human-readable identifier for the credit note
        :param idempotency_key: idempotency_key is an optional key used to prevent duplicate credit note creation
        :param line_items: line_items contains the individual line items that make up this credit note (minimum 1 required)
        :param memo: memo is an optional free-text field for additional notes about the credit note
        :param metadata:
        :param process_credit_note: process_credit_note is a flag to process the credit note after creation
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DtoCreateCreditNoteRequest(
            credit_note_number=credit_note_number,
            idempotency_key=idempotency_key,
            invoice_id=invoice_id,
            line_items=utils.get_pydantic_model(
                line_items, Optional[List[models.DtoCreateCreditNoteLineItemRequest]]
            ),
            memo=memo,
            metadata=metadata,
            process_credit_note=process_credit_note,
            reason=reason,
        )

        req = self._build_request(
            method="POST",
            path="/creditnotes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.DtoCreateCreditNoteRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createCreditNote",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.DtoCreditNoteResponse, http_res)
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def create_credit_note_async(
        self,
        *,
        invoice_id: str,
        reason: models.CreditNoteReason,
        credit_note_number: Optional[str] = None,
        idempotency_key: Optional[str] = None,
        line_items: Optional[
            Union[
                List[models.DtoCreateCreditNoteLineItemRequest],
                List[models.DtoCreateCreditNoteLineItemRequestTypedDict],
            ]
        ] = None,
        memo: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        process_credit_note: Optional[bool] = True,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoCreditNoteResponse:
        r"""Create credit note

        Use when issuing a refund or adjustment (e.g. customer dispute or proration). Links to an invoice; create as draft then finalize.

        :param invoice_id: invoice_id is the unique identifier of the invoice this credit note is applied to
        :param reason:
        :param credit_note_number: credit_note_number is an optional human-readable identifier for the credit note
        :param idempotency_key: idempotency_key is an optional key used to prevent duplicate credit note creation
        :param line_items: line_items contains the individual line items that make up this credit note (minimum 1 required)
        :param memo: memo is an optional free-text field for additional notes about the credit note
        :param metadata:
        :param process_credit_note: process_credit_note is a flag to process the credit note after creation
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DtoCreateCreditNoteRequest(
            credit_note_number=credit_note_number,
            idempotency_key=idempotency_key,
            invoice_id=invoice_id,
            line_items=utils.get_pydantic_model(
                line_items, Optional[List[models.DtoCreateCreditNoteLineItemRequest]]
            ),
            memo=memo,
            metadata=metadata,
            process_credit_note=process_credit_note,
            reason=reason,
        )

        req = self._build_request_async(
            method="POST",
            path="/creditnotes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.DtoCreateCreditNoteRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createCreditNote",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.DtoCreditNoteResponse, http_res)
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    def get_credit_note(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoCreditNoteResponse:
        r"""Get credit note

        Use when you need to load a single credit note (e.g. for display or reconciliation).

        :param id: Credit note ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetCreditNoteRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/creditnotes/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getCreditNote",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoCreditNoteResponse, http_res)
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def get_credit_note_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoCreditNoteResponse:
        r"""Get credit note

        Use when you need to load a single credit note (e.g. for display or reconciliation).

        :param id: Credit note ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetCreditNoteRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/creditnotes/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getCreditNote",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoCreditNoteResponse, http_res)
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    def process_credit_note(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoCreditNoteResponse:
        r"""Finalize credit note

        Use when locking a draft credit note and applying the credit (e.g. after approval). Once finalized, applied per billing provider.

        :param id: Credit note ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ProcessCreditNoteRequest(
            id=id,
        )

        req = self._build_request(
            method="POST",
            path="/creditnotes/{id}/finalize",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="processCreditNote",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoCreditNoteResponse, http_res)
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def process_credit_note_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoCreditNoteResponse:
        r"""Finalize credit note

        Use when locking a draft credit note and applying the credit (e.g. after approval). Once finalized, applied per billing provider.

        :param id: Credit note ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ProcessCreditNoteRequest(
            id=id,
        )

        req = self._build_request_async(
            method="POST",
            path="/creditnotes/{id}/finalize",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="processCreditNote",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoCreditNoteResponse, http_res)
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    def void_credit_note(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoCreditNoteResponse:
        r"""Void credit note

        Use when cancelling a draft credit note (e.g. created by mistake). Only draft credit notes can be voided.

        :param id: Credit note ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.VoidCreditNoteRequest(
            id=id,
        )

        req = self._build_request(
            method="POST",
            path="/creditnotes/{id}/void",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="voidCreditNote",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoCreditNoteResponse, http_res)
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def void_credit_note_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoCreditNoteResponse:
        r"""Void credit note

        Use when cancelling a draft credit note (e.g. created by mistake). Only draft credit notes can be voided.

        :param id: Credit note ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.VoidCreditNoteRequest(
            id=id,
        )

        req = self._build_request_async(
            method="POST",
            path="/creditnotes/{id}/void",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="voidCreditNote",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoCreditNoteResponse, http_res)
        if utils.match_response(
            http_res, ["400", "401", "403", "404"], "application/json"
        ):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice.types import BaseModel
from typing_extensions import TypedDict


class DtoCreateGroupRequestTypedDict(TypedDict):
    entity_type: str
    lookup_key: str
    name: str


class DtoCreateGroupRequest(BaseModel):
    entity_type: str

    lookup_key: str

    name: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_subscriptionusagebymetersresponse import (
    DtoSubscriptionUsageByMetersResponse,
    DtoSubscriptionUsageByMetersResponseTypedDict,
)
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoGetUsageBySubscriptionResponseTypedDict(TypedDict):
    amount: NotRequired[float]
    charges: NotRequired[List[DtoSubscriptionUsageByMetersResponseTypedDict]]
    commitment_amount: NotRequired[float]
    commitment_utilized: NotRequired[float]
    r"""Amount of commitment used"""
    currency: NotRequired[str]
    display_amount: NotRequired[str]
    end_time: NotRequired[str]
    has_overage: NotRequired[bool]
    r"""Whether any usage exceeded commitment"""
    overage_amount: NotRequired[float]
    r"""Amount charged at overage rate"""
    overage_factor: NotRequired[float]
    start_time: NotRequired[str]


class DtoGetUsageBySubscriptionResponse(BaseModel):
    amount: Optional[float] = None

    charges: Optional[List[DtoSubscriptionUsageByMetersResponse]] = None

    commitment_amount: Optional[float] = None

    commitment_utilized: Optional[float] = None
    r"""Amount of commitment used"""

    currency: Optional[str] = None

    display_amount: Optional[str] = None

    end_time: Optional[str] = None

    has_overage: Optional[bool] = None
    r"""Whether any usage exceeded commitment"""

    overage_amount: Optional[float] = None
    r"""Amount charged at overage rate"""

    overage_factor: Optional[float] = None

    start_time: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount",
                "charges",
                "commitment_amount",
                "commitment_utilized",
                "currency",
                "display_amount",
                "end_time",
                "has_overage",
                "overage_amount",
                "overage_factor",
                "start_time",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .paymentdestinationtype import PaymentDestinationType
from .paymentgatewaytype import PaymentGatewayType
from .paymentmethodtype import PaymentMethodType
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreatePaymentRequestTypedDict(TypedDict):
    amount: str
    currency: str
    destination_id: str
    destination_type: PaymentDestinationType
    payment_method_type: PaymentMethodType
    cancel_url: NotRequired[str]
    idempotency_key: NotRequired[str]
    metadata: NotRequired[Dict[str, str]]
    payment_gateway: NotRequired[PaymentGatewayType]
    payment_method_id: NotRequired[str]
    process_payment: NotRequired[bool]
    save_card_and_make_default: NotRequired[bool]
    success_url: NotRequired[str]


class DtoCreatePaymentRequest(BaseModel):
    amount: str

    currency: str

    destination_id: str

    destination_type: PaymentDestinationType

    payment_method_type: PaymentMethodType

    cancel_url: Optional[str] = None

    idempotency_key: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None

    payment_gateway: Optional[PaymentGatewayType] = None

    payment_method_id: Optional[str] = None

    process_payment: Optional[bool] = True

    save_card_and_make_default: Optional[bool] = False

    success_url: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "cancel_url",
                "idempotency_key",
                "metadata",
                "payment_gateway",
                "payment_method_id",
                "process_payment",
                "save_card_and_make_default",
                "success_url",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingcadence import BillingCadence
from .billingcycle import BillingCycle
from .billingperiod import BillingPeriod
from .dto_couponassociationresponse import (
    DtoCouponAssociationResponse,
    DtoCouponAssociationResponseTypedDict,
)
from .dto_creditgrantresponse import (
    DtoCreditGrantResponse,
    DtoCreditGrantResponseTypedDict,
)
from .dto_customerresponse import DtoCustomerResponse, DtoCustomerResponseTypedDict
from .dto_planresponse import DtoPlanResponse, DtoPlanResponseTypedDict
from .dto_subscriptionphaseresponse import (
    DtoSubscriptionPhaseResponse,
    DtoSubscriptionPhaseResponseTypedDict,
)
from .pausestatus import PauseStatus
from .paymentterms import PaymentTerms
from .prorationbehavior import ProrationBehavior
from .status import Status
from .subscription_subscriptionlineitem import (
    SubscriptionSubscriptionLineItem,
    SubscriptionSubscriptionLineItemTypedDict,
)
from .subscription_subscriptionpause import (
    SubscriptionSubscriptionPause,
    SubscriptionSubscriptionPauseTypedDict,
)
from .subscriptionstatus import SubscriptionStatus
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional, TYPE_CHECKING
from typing_extensions import NotRequired, TypedDict

if TYPE_CHECKING:
    from .dto_invoiceresponse import DtoInvoiceResponse, DtoInvoiceResponseTypedDict


class DtoSubscriptionResponseTypedDict(TypedDict):
    active_pause_id: NotRequired[str]
    r"""ActivePauseID references the current active pause configuration
    This will be null if no pause is active or scheduled
    """
    billing_anchor: NotRequired[str]
    r"""BillingAnchor is the reference point that aligns future billing cycle dates.
    It sets the day of week for week intervals, the day of month for month and year intervals,
    and the month of year for year intervals. The timestamp is in UTC format.
    """
    billing_cadence: NotRequired[BillingCadence]
    billing_cycle: NotRequired[BillingCycle]
    billing_period: NotRequired[BillingPeriod]
    billing_period_count: NotRequired[int]
    r"""BillingPeriodCount is the total number units of the billing period."""
    cancel_at: NotRequired[str]
    r"""CancelAt is the date the subscription will be canceled"""
    cancel_at_period_end: NotRequired[bool]
    r"""CancelAtPeriodEnd is whether the subscription was canceled at the end of the current period"""
    cancelled_at: NotRequired[str]
    r"""CanceledAt is the date the subscription was canceled"""
    collection_method: NotRequired[str]
    r"""CollectionMethod determines how invoices are collected"""
    commitment_amount: NotRequired[str]
    r"""CommitmentAmount is the minimum amount a customer commits to paying for a billing period"""
    commitment_duration: NotRequired[BillingPeriod]
    coupon_associations: NotRequired[List[DtoCouponAssociationResponseTypedDict]]
    r"""CouponAssociations are the coupon associations for this subscription"""
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    credit_grants: NotRequired[List[DtoCreditGrantResponseTypedDict]]
    r"""Credit grants are the credit grants for this subscription"""
    currency: NotRequired[str]
    r"""Currency is the currency of the subscription in lowercase 3 digit ISO codes"""
    current_period_end: NotRequired[str]
    r"""CurrentPeriodEnd is the end of the current period that the subscription has been invoiced for.
    At the end of this period, a new invoice will be created.
    """
    current_period_start: NotRequired[str]
    r"""CurrentPeriodStart is the end of the current period that the subscription has been invoiced for.
    At the end of this period, a new invoice will be created.
    """
    customer: NotRequired[DtoCustomerResponseTypedDict]
    r"""Customer response object containing all customer information"""
    customer_id: NotRequired[str]
    r"""CustomerID is the identifier for the customer in our system"""
    customer_timezone: NotRequired[str]
    enable_true_up: NotRequired[bool]
    end_date: NotRequired[str]
    r"""EndDate is the end date of the subscription"""
    environment_id: NotRequired[str]
    r"""EnvironmentID is the environment identifier for the subscription"""
    gateway_payment_method_id: NotRequired[str]
    r"""GatewayPaymentMethodID is the gateway payment method ID for this subscription"""
    id: NotRequired[str]
    r"""ID is the unique identifier for the subscription"""
    invoicing_customer_id: NotRequired[str]
    r"""InvoicingCustomerID is the customer ID to use for invoicing
    This can differ from the subscription customer (e.g., parent company invoicing for child company)
    """
    latest_invoice: NotRequired["DtoInvoiceResponseTypedDict"]
    line_items: NotRequired[List[SubscriptionSubscriptionLineItemTypedDict]]
    lookup_key: NotRequired[str]
    r"""LookupKey is the key used to lookup the subscription in our system"""
    metadata: NotRequired[Dict[str, str]]
    overage_factor: NotRequired[str]
    r"""OverageFactor is a multiplier applied to usage beyond the commitment amount"""
    parent_subscription_id: NotRequired[str]
    r"""ParentSubscriptionID is the parent subscription ID for hierarchy (e.g. child subscription under a parent)"""
    pause_status: NotRequired[PauseStatus]
    pauses: NotRequired[List[SubscriptionSubscriptionPauseTypedDict]]
    payment_behavior: NotRequired[str]
    r"""PaymentBehavior determines how subscription payments are handled"""
    payment_terms: NotRequired[PaymentTerms]
    phases: NotRequired[List[DtoSubscriptionPhaseResponseTypedDict]]
    r"""Phases are the subscription phases for this subscription"""
    plan: NotRequired[DtoPlanResponseTypedDict]
    plan_id: NotRequired[str]
    r"""PlanID is the identifier for the plan in our system"""
    proration_behavior: NotRequired[ProrationBehavior]
    start_date: NotRequired[str]
    r"""StartDate is the start date of the subscription"""
    status: NotRequired[Status]
    subscription_status: NotRequired[SubscriptionStatus]
    tenant_id: NotRequired[str]
    trial_end: NotRequired[str]
    r"""TrialEnd is the end date of the trial period"""
    trial_start: NotRequired[str]
    r"""TrialStart is the start date of the trial period"""
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]
    version: NotRequired[int]
    r"""Version is used for optimistic locking"""


class DtoSubscriptionResponse(BaseModel):
    active_pause_id: Optional[str] = None
    r"""ActivePauseID references the current active pause configuration
    This will be null if no pause is active or scheduled
    """

    billing_anchor: Optional[str] = None
    r"""BillingAnchor is the reference point that aligns future billing cycle dates.
    It sets the day of week for week intervals, the day of month for month and year intervals,
    and the month of year for year intervals. The timestamp is in UTC format.
    """

    billing_cadence: Optional[BillingCadence] = None

    billing_cycle: Optional[BillingCycle] = None

    billing_period: Optional[BillingPeriod] = None

    billing_period_count: Optional[int] = None
    r"""BillingPeriodCount is the total number units of the billing period."""

    cancel_at: Optional[str] = None
    r"""CancelAt is the date the subscription will be canceled"""

    cancel_at_period_end: Optional[bool] = None
    r"""CancelAtPeriodEnd is whether the subscription was canceled at the end of the current period"""

    cancelled_at: Optional[str] = None
    r"""CanceledAt is the date the subscription was canceled"""

    collection_method: Optional[str] = None
    r"""CollectionMethod determines how invoices are collected"""

    commitment_amount: Optional[str] = None
    r"""CommitmentAmount is the minimum amount a customer commits to paying for a billing period"""

    commitment_duration: Optional[BillingPeriod] = None

    coupon_associations: Optional[List[DtoCouponAssociationResponse]] = None
    r"""CouponAssociations are the coupon associations for this subscription"""

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    credit_grants: Optional[List[DtoCreditGrantResponse]] = None
    r"""Credit grants are the credit grants for this subscription"""

    currency: Optional[str] = None
    r"""Currency is the currency of the subscription in lowercase 3 digit ISO codes"""

    current_period_end: Optional[str] = None
    r"""CurrentPeriodEnd is the end of the current period that the subscription has been invoiced for.
    At the end of this period, a new invoice will be created.
    """

    current_period_start: Optional[str] = None
    r"""CurrentPeriodStart is the end of the current period that the subscription has been invoiced for.
    At the end of this period, a new invoice will be created.
    """

    customer: Optional[DtoCustomerResponse] = None
    r"""Customer response object containing all customer information"""

    customer_id: Optional[str] = None
    r"""CustomerID is the identifier for the customer in our system"""

    customer_timezone: Optional[str] = None

    enable_true_up: Optional[bool] = None

    end_date: Optional[str] = None
    r"""EndDate is the end date of the subscription"""

    environment_id: Optional[str] = None
    r"""EnvironmentID is the environment identifier for the subscription"""

    gateway_payment_method_id: Optional[str] = None
    r"""GatewayPaymentMethodID is the gateway payment method ID for this subscription"""

    id: Optional[str] = None
    r"""ID is the unique identifier for the subscription"""

    invoicing_customer_id: Optional[str] = None
    r"""InvoicingCustomerID is the customer ID to use for invoicing
    This can differ from the subscription customer (e.g., parent company invoicing for child company)
    """

    latest_invoice: Optional["DtoInvoiceResponse"] = None

    line_items: Optional[List[SubscriptionSubscriptionLineItem]] = None

    lookup_key: Optional[str] = None
    r"""LookupKey is the key used to lookup the subscription in our system"""

    metadata: Optional[Dict[str, str]] = None

    overage_factor: Optional[str] = None
    r"""OverageFactor is a multiplier applied to usage beyond the commitment amount"""

    parent_subscription_id: Optional[str] = None
    r"""ParentSubscriptionID is the parent subscription ID for hierarchy (e.g. child subscription under a parent)"""

    pause_status: Optional[PauseStatus] = None

    pauses: Optional[List[SubscriptionSubscriptionPause]] = None

    payment_behavior: Optional[str] = None
    r"""PaymentBehavior determines how subscription payments are handled"""

    payment_terms: Optional[PaymentTerms] = None

    phases: Optional[List[DtoSubscriptionPhaseResponse]] = None
    r"""Phases are the subscription phases for this subscription"""

    plan: Optional[DtoPlanResponse] = None

    plan_id: Optional[str] = None
    r"""PlanID is the identifier for the plan in our system"""

    proration_behavior: Optional[ProrationBehavior] = None

    start_date: Optional[str] = None
    r"""StartDate is the start date of the subscription"""

    status: Optional[Status] = None

    subscription_status: Optional[SubscriptionStatus] = None

    tenant_id: Optional[str] = None

    trial_end: Optional[str] = None
    r"""TrialEnd is the end date of the trial period"""

    trial_start: Optional[str] = None
    r"""TrialStart is the start date of the trial period"""

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    version: Optional[int] = None
    r"""Version is used for optimistic locking"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "active_pause_id",
                "billing_anchor",
                "billing_cadence",
                "billing_cycle",
                "billing_period",
                "billing_period_count",
                "cancel_at",
                "cancel_at_period_end",
                "cancelled_at",
                "collection_method",
                "commitment_amount",
                "commitment_duration",
                "coupon_associations",
                "created_at",
                "created_by",
                "credit_grants",
                "currency",
                "current_period_end",
                "current_period_start",
                "customer",
                "customer_id",
                "customer_timezone",
                "enable_true_up",
                "end_date",
                "environment_id",
                "gateway_payment_method_id",
                "id",
                "invoicing_customer_id",
                "latest_invoice",
                "line_items",
                "lookup_key",
                "metadata",
                "overage_factor",
                "parent_subscription_id",
                "pause_status",
                "pauses",
                "payment_behavior",
                "payment_terms",
                "phases",
                "plan",
                "plan_id",
                "proration_behavior",
                "start_date",
                "status",
                "subscription_status",
                "tenant_id",
                "trial_end",
                "trial_start",
                "updated_at",
                "updated_by",
                "version",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .alertentitytype import AlertEntityType
from .alertstate import AlertState
from .alerttype import AlertType
from .filtercondition import FilterCondition, FilterConditionTypedDict
from .sortcondition import SortCondition, SortConditionTypedDict
from .status import Status
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


AlertLogFilterOrder = Literal[
    "asc",
    "desc",
]


class AlertLogFilterTypedDict(TypedDict):
    alert_status: NotRequired[AlertState]
    alert_type: NotRequired[AlertType]
    customer_id: NotRequired[str]
    end_time: NotRequired[str]
    entity_id: NotRequired[str]
    entity_type: NotRequired[AlertEntityType]
    expand: NotRequired[str]
    filters: NotRequired[List[FilterConditionTypedDict]]
    r"""filters allows complex filtering based on multiple fields"""
    limit: NotRequired[int]
    offset: NotRequired[int]
    order: NotRequired[AlertLogFilterOrder]
    sort: NotRequired[List[SortConditionTypedDict]]
    start_time: NotRequired[str]
    status: NotRequired[Status]


class AlertLogFilter(BaseModel):
    alert_status: Optional[AlertState] = None

    alert_type: Optional[AlertType] = None

    customer_id: Optional[str] = None

    end_time: Optional[str] = None

    entity_id: Optional[str] = None

    entity_type: Optional[AlertEntityType] = None

    expand: Optional[str] = None

    filters: Optional[List[FilterCondition]] = None
    r"""filters allows complex filtering based on multiple fields"""

    limit: Optional[int] = None

    offset: Optional[int] = None

    order: Optional[AlertLogFilterOrder] = None

    sort: Optional[List[SortCondition]] = None

    start_time: Optional[str] = None

    status: Optional[Status] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "alert_status",
                "alert_type",
                "customer_id",
                "end_time",
                "entity_id",
                "entity_type",
                "expand",
                "filters",
                "limit",
                "offset",
                "order",
                "sort",
                "start_time",
                "status",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice.types import UnrecognizedStr
from typing import Literal, Union


FeatureType = Union[
    Literal[
        "metered",
        "boolean",
        "static",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .taxrateentitytype import TaxRateEntityType
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreateTaxAssociationRequestTypedDict(TypedDict):
    tax_rate_code: str
    auto_apply: NotRequired[bool]
    currency: NotRequired[str]
    entity_id: NotRequired[str]
    entity_type: NotRequired[TaxRateEntityType]
    external_customer_id: NotRequired[str]
    metadata: NotRequired[Dict[str, str]]
    priority: NotRequired[int]


class DtoCreateTaxAssociationRequest(BaseModel):
    tax_rate_code: str

    auto_apply: Optional[bool] = None

    currency: Optional[str] = None

    entity_id: Optional[str] = None

    entity_type: Optional[TaxRateEntityType] = None

    external_customer_id: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None

    priority: Optional[int] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "auto_apply",
                "currency",
                "entity_id",
                "entity_type",
                "external_customer_id",
                "metadata",
                "priority",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice.types import BaseModel, UNSET_SENTINEL
from flexprice.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


ListPaymentsOrder = Literal[
    "asc",
    "desc",
]


ListPaymentsStatus = Literal[
    "published",
    "deleted",
    "archived",
]


class ListPaymentsRequestTypedDict(TypedDict):
    currency: NotRequired[str]
    destination_id: NotRequired[str]
    destination_type: NotRequired[str]
    end_time: NotRequired[str]
    expand: NotRequired[str]
    gateway_payment_id: NotRequired[str]
    gateway_tracking_id: NotRequired[str]
    r"""For filtering by gateway tracking ID"""
    limit: NotRequired[int]
    offset: NotRequired[int]
    order: NotRequired[ListPaymentsOrder]
    payment_gateway: NotRequired[str]
    payment_ids: NotRequired[List[str]]
    payment_method_type: NotRequired[str]
    payment_status: NotRequired[str]
    sort: NotRequired[str]
    start_time: NotRequired[str]
    status: NotRequired[ListPaymentsStatus]


class ListPaymentsRequest(BaseModel):
    currency: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    destination_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    destination_type: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    end_time: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    expand: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    gateway_payment_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    gateway_tracking_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""For filtering by gateway tracking ID"""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    offset: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    order: Annotated[
        Optional[ListPaymentsOrder],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    payment_gateway: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    payment_ids: Annotated[
        Optional[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    payment_method_type: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    payment_status: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    sort: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    start_time: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    status: Annotated[
        Optional[ListPaymentsStatus],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "currency",
                "destination_id",
                "destination_type",
                "end_time",
                "expand",
                "gateway_payment_id",
                "gateway_tracking_id",
                "limit",
                "offset",
                "order",
                "payment_gateway",
                "payment_ids",
                "payment_method_type",
                "payment_status",
                "sort",
                "start_time",
                "status",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .couponcadence import CouponCadence
from .coupontype import CouponType
from .status import Status
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCouponResponseTypedDict(TypedDict):
    amount_off: NotRequired[str]
    cadence: NotRequired[CouponCadence]
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    currency: NotRequired[str]
    duration_in_periods: NotRequired[int]
    environment_id: NotRequired[str]
    id: NotRequired[str]
    max_redemptions: NotRequired[int]
    metadata: NotRequired[Dict[str, str]]
    name: NotRequired[str]
    percentage_off: NotRequired[str]
    redeem_after: NotRequired[str]
    redeem_before: NotRequired[str]
    rules: NotRequired[Dict[str, Any]]
    status: NotRequired[Status]
    tenant_id: NotRequired[str]
    total_redemptions: NotRequired[int]
    type: NotRequired[CouponType]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]


class DtoCouponResponse(BaseModel):
    amount_off: Optional[str] = None

    cadence: Optional[CouponCadence] = None

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    currency: Optional[str] = None

    duration_in_periods: Optional[int] = None

    environment_id: Optional[str] = None

    id: Optional[str] = None

    max_redemptions: Optional[int] = None

    metadata: Optional[Dict[str, str]] = None

    name: Optional[str] = None

    percentage_off: Optional[str] = None

    redeem_after: Optional[str] = None

    redeem_before: Optional[str] = None

    rules: Optional[Dict[str, Any]] = None

    status: Optional[Status] = None

    tenant_id: Optional[str] = None

    total_redemptions: Optional[int] = None

    type: Optional[CouponType] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount_off",
                "cadence",
                "created_at",
                "created_by",
                "currency",
                "duration_in_periods",
                "environment_id",
                "id",
                "max_redemptions",
                "metadata",
                "name",
                "percentage_off",
                "redeem_after",
                "redeem_before",
                "rules",
                "status",
                "tenant_id",
                "total_redemptions",
                "type",
                "updated_at",
                "updated_by",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .transactionreason import TransactionReason
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoTopUpWalletRequestTypedDict(TypedDict):
    transaction_reason: TransactionReason
    amount: NotRequired[str]
    r"""amount is the amount in the currency of the wallet to be added
    NOTE: this is not the number of credits to add, but the amount in the currency
    amount = credits_to_add * conversion_rate
    if both amount and credits_to_add are provided, amount will be ignored
    ex if the wallet has a conversion_rate of 2 then adding an amount of
    10 USD in the wallet wil add 5 credits in the wallet
    """
    credits_to_add: NotRequired[str]
    r"""credits_to_add is the number of credits to add to the wallet"""
    description: NotRequired[str]
    r"""description to add any specific details about the transaction"""
    expiry_date_utc: NotRequired[str]
    r"""expiry_date_utc is the expiry date in UTC timezone
    ex 2025-01-01 00:00:00 UTC
    """
    idempotency_key: NotRequired[str]
    r"""idempotency_key is a unique key for the transaction"""
    metadata: NotRequired[Dict[str, str]]
    priority: NotRequired[int]
    r"""priority is the priority of the transaction
    lower number means higher priority
    default is nil which means no priority at all
    """


class DtoTopUpWalletRequest(BaseModel):
    transaction_reason: TransactionReason

    amount: Optional[str] = None
    r"""amount is the amount in the currency of the wallet to be added
    NOTE: this is not the number of credits to add, but the amount in the currency
    amount = credits_to_add * conversion_rate
    if both amount and credits_to_add are provided, amount will be ignored
    ex if the wallet has a conversion_rate of 2 then adding an amount of
    10 USD in the wallet wil add 5 credits in the wallet
    """

    credits_to_add: Optional[str] = None
    r"""credits_to_add is the number of credits to add to the wallet"""

    description: Optional[str] = None
    r"""description to add any specific details about the transaction"""

    expiry_date_utc: Optional[str] = None
    r"""expiry_date_utc is the expiry date in UTC timezone
    ex 2025-01-01 00:00:00 UTC
    """

    idempotency_key: Optional[str] = None
    r"""idempotency_key is a unique key for the transaction"""

    metadata: Optional[Dict[str, str]] = None

    priority: Optional[int] = None
    r"""priority is the priority of the transaction
    lower number means higher priority
    default is nil which means no priority at all
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount",
                "credits_to_add",
                "description",
                "expiry_date_utc",
                "idempotency_key",
                "metadata",
                "priority",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from flexprice import models, utils
from flexprice._hooks import HookContext
from flexprice.types import OptionalNullable, UNSET
from flexprice.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional


class TaxRates(BaseSDK):
    def get_tax_rates(
        self,
        *,
        end_time: Optional[str] = None,
        expand: Optional[str] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.GetTaxRatesOrder] = None,
        scope: Optional[models.Scope] = None,
        start_time: Optional[str] = None,
        status: Optional[models.GetTaxRatesStatus] = None,
        taxrate_codes: Optional[List[str]] = None,
        taxrate_ids: Optional[List[str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.DtoTaxRateResponse]:
        r"""Get tax rates

        Use when listing tax rates (e.g. tax config UI). Returns tax rates with optional filters.

        :param end_time:
        :param expand:
        :param limit:
        :param offset:
        :param order:
        :param scope:
        :param start_time:
        :param status:
        :param taxrate_codes:
        :param taxrate_ids:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetTaxRatesRequest(
            end_time=end_time,
            expand=expand,
            limit=limit,
            offset=offset,
            order=order,
            scope=scope,
            start_time=start_time,
            status=status,
            taxrate_codes=taxrate_codes,
            taxrate_ids=taxrate_ids,
        )

        req = self._build_request(
            method="GET",
            path="/taxes/rates",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getTaxRates",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(List[models.DtoTaxRateResponse], http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def get_tax_rates_async(
        self,
        *,
        end_time: Optional[str] = None,
        expand: Optional[str] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.GetTaxRatesOrder] = None,
        scope: Optional[models.Scope] = None,
        start_time: Optional[str] = None,
        status: Optional[models.GetTaxRatesStatus] = None,
        taxrate_codes: Optional[List[str]] = None,
        taxrate_ids: Optional[List[str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.DtoTaxRateResponse]:
        r"""Get tax rates

        Use when listing tax rates (e.g. tax config UI). Returns tax rates with optional filters.

        :param end_time:
        :param expand:
        :param limit:
        :param offset:
        :param order:
        :param scope:
        :param start_time:
        :param status:
        :param taxrate_codes:
        :param taxrate_ids:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetTaxRatesRequest(
            end_time=end_time,
            expand=expand,
            limit=limit,
            offset=offset,
            order=order,
            scope=scope,
            start_time=start_time,
            status=status,
            taxrate_codes=taxrate_codes,
            taxrate_ids=taxrate_ids,
        )

        req = self._build_request_async(
            method="GET",
            path="/taxes/rates",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getTaxRates",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(List[models.DtoTaxRateResponse], http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    def create_tax_rate(
        self,
        *,
        code: str,
        name: str,
        description: Optional[str] = None,
        fixed_value: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        percentage_value: Optional[str] = None,
        scope: Optional[models.TaxRateScope] = None,
        tax_rate_type: Optional[models.TaxRateType] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoTaxRateResponse:
        r"""Create a tax rate

        Use when defining a new tax rate (e.g. VAT or sales tax) for use in invoices. Attach to customers or products via tax associations.

        :param code: code is the unique alphanumeric case sensitive identifier for the tax rate (required)
        :param name: name is the human-readable name for the tax rate (required)
        :param description: description is an optional text description providing details about the tax rate
        :param fixed_value: fixed_value is the fixed monetary amount when tax_rate_type is \"fixed\"
        :param metadata: metadata contains additional key-value pairs for storing extra information
        :param percentage_value: percentage_value is the percentage value (0-100) when tax_rate_type is \"percentage\"
        :param scope:
        :param tax_rate_type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DtoCreateTaxRateRequest(
            code=code,
            description=description,
            fixed_value=fixed_value,
            metadata=metadata,
            name=name,
            percentage_value=percentage_value,
            scope=scope,
            tax_rate_type=tax_rate_type,
        )

        req = self._build_request(
            method="POST",
            path="/taxes/rates",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.DtoCreateTaxRateRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createTaxRate",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.DtoTaxRateResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def create_tax_rate_async(
        self,
        *,
        code: str,
        name: str,
        description: Optional[str] = None,
        fixed_value: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        percentage_value: Optional[str] = None,
        scope: Optional[models.TaxRateScope] = None,
        tax_rate_type: Optional[models.TaxRateType] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoTaxRateResponse:
        r"""Create a tax rate

        Use when defining a new tax rate (e.g. VAT or sales tax) for use in invoices. Attach to customers or products via tax associations.

        :param code: code is the unique alphanumeric case sensitive identifier for the tax rate (required)
        :param name: name is the human-readable name for the tax rate (required)
        :param description: description is an optional text description providing details about the tax rate
        :param fixed_value: fixed_value is the fixed monetary amount when tax_rate_type is \"fixed\"
        :param metadata: metadata contains additional key-value pairs for storing extra information
        :param percentage_value: percentage_value is the percentage value (0-100) when tax_rate_type is \"percentage\"
        :param scope:
        :param tax_rate_type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DtoCreateTaxRateRequest(
            code=code,
            description=description,
            fixed_value=fixed_value,
            metadata=metadata,
            name=name,
            percentage_value=percentage_value,
            scope=scope,
            tax_rate_type=tax_rate_type,
        )

        req = self._build_request_async(
            method="POST",
            path="/taxes/rates",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.DtoCreateTaxRateRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createTaxRate",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.DtoTaxRateResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    def get_tax_rate(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoTaxRateResponse:
        r"""Get a tax rate

        Use when you need to load a single tax rate (e.g. for display or when creating an association).

        :param id: Tax rate ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetTaxRateRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/taxes/rates/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getTaxRate",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoTaxRateResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def get_tax_rate_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoTaxRateResponse:
        r"""Get a tax rate

        Use when you need to load a single tax rate (e.g. for display or when creating an association).

        :param id: Tax rate ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetTaxRateRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/taxes/rates/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getTaxRate",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoTaxRateResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    def update_tax_rate(
        self,
        *,
        id: str,
        code: Optional[str] = None,
        description: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        name: Optional[str] = None,
        tax_rate_status: Optional[models.TaxRateStatus] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoTaxRateResponse:
        r"""Update a tax rate

        Use when changing a tax rate (e.g. rate value or name). Request body contains the fields to update.

        :param id: Tax rate ID
        :param code: code is the updated unique alphanumeric identifier for the tax rate
        :param description: description is the updated text description for the tax rate
        :param metadata: metadata contains updated key-value pairs that will replace existing metadata
        :param name: name is the updated human-readable name for the tax rate
        :param tax_rate_status:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateTaxRateRequest(
            id=id,
            body=models.DtoUpdateTaxRateRequest(
                code=code,
                description=description,
                metadata=metadata,
                name=name,
                tax_rate_status=tax_rate_status,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/taxes/rates/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body, False, False, "json", models.DtoUpdateTaxRateRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateTaxRate",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoTaxRateResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def update_tax_rate_async(
        self,
        *,
        id: str,
        code: Optional[str] = None,
        description: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        name: Optional[str] = None,
        tax_rate_status: Optional[models.TaxRateStatus] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoTaxRateResponse:
        r"""Update a tax rate

        Use when changing a tax rate (e.g. rate value or name). Request body contains the fields to update.

        :param id: Tax rate ID
        :param code: code is the updated unique alphanumeric identifier for the tax rate
        :param description: description is the updated text description for the tax rate
        :param metadata: metadata contains updated key-value pairs that will replace existing metadata
        :param name: name is the updated human-readable name for the tax rate
        :param tax_rate_status:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateTaxRateRequest(
            id=id,
            body=models.DtoUpdateTaxRateRequest(
                code=code,
                description=description,
                metadata=metadata,
                name=name,
                tax_rate_status=tax_rate_status,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/taxes/rates/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body, False, False, "json", models.DtoUpdateTaxRateRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateTaxRate",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoTaxRateResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    def delete_tax_rate(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Delete a tax rate

        Use when retiring a tax rate (e.g. no longer applicable). Fails if still referenced by associations.

        :param id: Tax rate ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeleteTaxRateRequest(
            id=id,
        )

        req = self._build_request(
            method="DELETE",
            path="/taxes/rates/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deleteTaxRate",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def delete_tax_rate_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Delete a tax rate

        Use when retiring a tax rate (e.g. no longer applicable). Fails if still referenced by associations.

        :param id: Tax rate ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeleteTaxRateRequest(
            id=id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/taxes/rates/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deleteTaxRate",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )



#!/usr/bin/env python3
"""
FlexPrice Python SDK example – sync client, create customer and ingest event.
Set FLEXPRICE_API_KEY (and optionally FLEXPRICE_API_HOST) in .env or environment.
"""

import os
import time
from dotenv import load_dotenv

load_dotenv()

from flexprice import Flexprice

def main():
    api_key = os.getenv("FLEXPRICE_API_KEY")
    api_host = os.getenv("FLEXPRICE_API_HOST", "us.api.flexprice.io")

    if not api_key:
        raise SystemExit("Set FLEXPRICE_API_KEY in .env or environment")

    server_url = api_host if api_host.startswith(("http://", "https://")) else f"https://{api_host}"
    customer_id = f"sample-customer-{int(time.time())}"

    print("Starting FlexPrice Python SDK example...")
    print(f"Using API key: {api_key[:4]}...{api_key[-4:]}")

    try:
        with Flexprice(
            server_url=server_url,
            api_key_auth=api_key,
        ) as flexprice:
            result = flexprice.events.ingest_event(
                request={
                    "event_name": "Sample Event",
                    "external_customer_id": customer_id,
                    "properties": {"source": "python_app", "environment": "test"},
                    "source": "python_app",
                }
            )
            print("Event ingested:", result)
        print("Example completed successfully!")
    except Exception as e:
        print(f"Error: {e}")
        raise

if __name__ == "__main__":
    main()



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pausemode import PauseMode
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoPauseSubscriptionRequestTypedDict(TypedDict):
    r"""Request object for pausing an active subscription with various pause modes and options"""

    pause_mode: PauseMode
    dry_run: NotRequired[bool]
    r"""Whether to perform a dry run
    @Description If true, validates the request and shows impact without actually pausing the subscription
    @Example false
    """
    metadata: NotRequired[Dict[str, str]]
    r"""Additional metadata as key-value pairs
    @Description Optional metadata for storing additional information about the pause
    @Example {\"requested_by\": \"customer\", \"channel\": \"support_ticket\"}
    """
    pause_days: NotRequired[int]
    r"""Duration of the pause in days
    @Description Number of days to pause the subscription. Cannot be used together with pause_end. Must be greater than 0
    @Example 30
    """
    pause_end: NotRequired[str]
    r"""End date for the subscription pause
    @Description ISO 8601 timestamp when the pause should end. Cannot be used together with pause_days. Must be after pause_start
    @Example \"2024-02-15T00:00:00Z\" 
    """
    pause_start: NotRequired[str]
    r"""Start date for the subscription pause
    @Description ISO 8601 timestamp when the pause should begin. Required when pause_mode is \"scheduled\" 
    @Example \"2024-01-15T00:00:00Z\" 
    """
    reason: NotRequired[str]
    r"""Reason for pausing the subscription
    @Description Optional reason for the pause. Maximum 255 characters
    @Example \"Customer requested temporary suspension\" 
    """


class DtoPauseSubscriptionRequest(BaseModel):
    r"""Request object for pausing an active subscription with various pause modes and options"""

    pause_mode: PauseMode

    dry_run: Optional[bool] = None
    r"""Whether to perform a dry run
    @Description If true, validates the request and shows impact without actually pausing the subscription
    @Example false
    """

    metadata: Optional[Dict[str, str]] = None
    r"""Additional metadata as key-value pairs
    @Description Optional metadata for storing additional information about the pause
    @Example {\"requested_by\": \"customer\", \"channel\": \"support_ticket\"}
    """

    pause_days: Optional[int] = None
    r"""Duration of the pause in days
    @Description Number of days to pause the subscription. Cannot be used together with pause_end. Must be greater than 0
    @Example 30
    """

    pause_end: Optional[str] = None
    r"""End date for the subscription pause
    @Description ISO 8601 timestamp when the pause should end. Cannot be used together with pause_days. Must be after pause_start
    @Example \"2024-02-15T00:00:00Z\" 
    """

    pause_start: Optional[str] = None
    r"""Start date for the subscription pause
    @Description ISO 8601 timestamp when the pause should begin. Required when pause_mode is \"scheduled\" 
    @Example \"2024-01-15T00:00:00Z\" 
    """

    reason: Optional[str] = None
    r"""Reason for pausing the subscription
    @Description Optional reason for the pause. Maximum 255 characters
    @Example \"Customer requested temporary suspension\" 
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["dry_run", "metadata", "pause_days", "pause_end", "pause_start", "reason"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoOverrideEntitlementRequestTypedDict(TypedDict):
    entitlement_id: str
    r"""EntitlementID references the plan/addon entitlement to override"""
    is_enabled: NotRequired[bool]
    r"""IsEnabled determines if the entitlement is enabled or disabled"""
    static_value: NotRequired[str]
    r"""StaticValue is the static value for static features"""
    usage_limit: NotRequired[int]
    r"""UsageLimit is the new usage limit (only these 3 fields can be overridden)
    For metered features, nil means unlimited usage
    """


class DtoOverrideEntitlementRequest(BaseModel):
    entitlement_id: str
    r"""EntitlementID references the plan/addon entitlement to override"""

    is_enabled: Optional[bool] = None
    r"""IsEnabled determines if the entitlement is enabled or disabled"""

    static_value: Optional[str] = None
    r"""StaticValue is the static value for static features"""

    usage_limit: Optional[int] = None
    r"""UsageLimit is the new usage limit (only these 3 fields can be overridden)
    For metered features, nil means unlimited usage
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["is_enabled", "static_value", "usage_limit"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingcadence import BillingCadence
from .billingperiod import BillingPeriod
from .filtercondition import FilterCondition, FilterConditionTypedDict
from .sortcondition import SortCondition, SortConditionTypedDict
from .status import Status
from .subscriptionstatus import SubscriptionStatus
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


SubscriptionFilterOrder = Literal[
    "asc",
    "desc",
]


class SubscriptionFilterTypedDict(TypedDict):
    active_at: NotRequired[str]
    r"""ActiveAt filters subscriptions that are active at the given time"""
    billing_cadence: NotRequired[List[BillingCadence]]
    r"""BillingCadence filters by billing cadence"""
    billing_period: NotRequired[List[BillingPeriod]]
    r"""BillingPeriod filters by billing period"""
    customer_id: NotRequired[str]
    r"""CustomerID filters by customer ID"""
    end_time: NotRequired[str]
    expand: NotRequired[str]
    external_customer_id: NotRequired[str]
    r"""ExternalCustomerID filters by external customer ID"""
    filters: NotRequired[List[FilterConditionTypedDict]]
    invoicing_customer_ids: NotRequired[List[str]]
    r"""InvoicingCustomerIDs filters by invoicing customer ID"""
    limit: NotRequired[int]
    offset: NotRequired[int]
    order: NotRequired[SubscriptionFilterOrder]
    parent_subscription_ids: NotRequired[List[str]]
    r"""ParentSubscriptionIDs filters by parent subscription IDs"""
    plan_id: NotRequired[str]
    r"""PlanID filters by plan ID"""
    sort: NotRequired[List[SortConditionTypedDict]]
    start_time: NotRequired[str]
    status: NotRequired[Status]
    subscription_ids: NotRequired[List[str]]
    subscription_status: NotRequired[List[SubscriptionStatus]]
    r"""SubscriptionStatus filters by subscription status"""
    with_line_items: NotRequired[bool]
    r"""WithLineItems includes line items in the response"""


class SubscriptionFilter(BaseModel):
    active_at: Optional[str] = None
    r"""ActiveAt filters subscriptions that are active at the given time"""

    billing_cadence: Optional[List[BillingCadence]] = None
    r"""BillingCadence filters by billing cadence"""

    billing_period: Optional[List[BillingPeriod]] = None
    r"""BillingPeriod filters by billing period"""

    customer_id: Optional[str] = None
    r"""CustomerID filters by customer ID"""

    end_time: Optional[str] = None

    expand: Optional[str] = None

    external_customer_id: Optional[str] = None
    r"""ExternalCustomerID filters by external customer ID"""

    filters: Optional[List[FilterCondition]] = None

    invoicing_customer_ids: Optional[List[str]] = None
    r"""InvoicingCustomerIDs filters by invoicing customer ID"""

    limit: Optional[int] = None

    offset: Optional[int] = None

    order: Optional[SubscriptionFilterOrder] = None

    parent_subscription_ids: Optional[List[str]] = None
    r"""ParentSubscriptionIDs filters by parent subscription IDs"""

    plan_id: Optional[str] = None
    r"""PlanID filters by plan ID"""

    sort: Optional[List[SortCondition]] = None

    start_time: Optional[str] = None

    status: Optional[Status] = None

    subscription_ids: Optional[List[str]] = None

    subscription_status: Optional[List[SubscriptionStatus]] = None
    r"""SubscriptionStatus filters by subscription status"""

    with_line_items: Optional[bool] = None
    r"""WithLineItems includes line items in the response"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "active_at",
                "billing_cadence",
                "billing_period",
                "customer_id",
                "end_time",
                "expand",
                "external_customer_id",
                "filters",
                "invoicing_customer_ids",
                "limit",
                "offset",
                "order",
                "parent_subscription_ids",
                "plan_id",
                "sort",
                "start_time",
                "status",
                "subscription_ids",
                "subscription_status",
                "with_line_items",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
